/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Staging repository detail view.
 *
 * @since 2.3
 */
NX.define('Nexus.staging.repository.RepositoryView', {
    extend: 'Ext.Panel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.Icons',
        'Nexus.staging.repository.SummaryView',
        'Nexus.staging.repository.activity.ActivityView',
        'Nexus.staging.repository.ContentView',
        'Nexus.staging.repository.MembersView'
    ],

    /**
     * @override
     */
    initComponent: function () {
        var self = this;

        self.summaryView = NX.create('Nexus.staging.repository.SummaryView');
        self.activityView = NX.create('Nexus.staging.repository.activity.ActivityView');
        self.contentView = NX.create('Nexus.staging.repository.ContentView');
        self.membersView = NX.create('Nexus.staging.repository.MembersView');

        self.allViews = [
            self.summaryView,
            self.activityView,
            self.contentView,
            self.membersView
        ];

        self.tabPanel = NX.create('Ext.TabPanel', {
            items: self.allViews,
            activeTab: 0,
            layoutOnTabChange: true
        });

        Ext.apply(self, {
            cls: 'nx-staging-repository-RepositoryView',
            header: true,
            border: false,
            layout: 'fit',

            items: self.tabPanel
        });

        self.constructor.superclass.initComponent.apply(self, arguments);

        // propagate 'hide' and 'deactivate' events to child views, 'show' and 'activate' handled when [re-]enabling views
        self.on('deactivate', function(panel) {
            //self.logDebug('Deactivated');
            Ext.each(self.allViews, function(view) {
                view.fireEvent('deactivate', view);
            });
        });
        self.on('hide', function(panel) {
            //self.logDebug('Hidden');
            Ext.each(self.allViews, function(view) {
                view.fireEvent('hide', view);
            });
        });

        //self.on('activate', function(panel) {
        //    self.logDebug('Activated');
        //});
        //self.on('show', function(panel) {
        //    self.logDebug('Shown');
        //});
    },

    /**
     * Enable specific views and disable all others.
     *
     * @param {Array} views   An array of view components to enable.
     *
     * @private
     */
    enableViews: function (views) {
        var self = this,
            lastActive = self.tabPanel.getActiveTab();

        if (views.length === 0) {
            throw 'At least one view must be enabled';
        }

        // Show all active views, hide others
        Ext.each(self.allViews, function (view) {
            if (views.indexOf(view) !== -1) {
                self.tabPanel.unhideTabStripItem(view);
            }
            else {
                self.tabPanel.hideTabStripItem(view);
            }
        });

        // If the active views does not contain the last active view, then reset back to first
        if (views.indexOf(lastActive) === -1) {
            self.tabPanel.setActiveTab(0);
        }
        else {
            // else inform the panel its showing and active
            lastActive.fireEvent('show', lastActive);
            lastActive.fireEvent('activate', lastActive);
        }
    },

    /**
     * Update detail display for repository record.
     *
     * @param repo
     */
    updateRecord: function (repo) {
        var self = this,
            iconCls;

        self.logGroup('Updating record');

        // summary and activity views are always enabled
        var views = [
            self.summaryView,
            self.activityView
        ];

        // extra views if repository is not transitioning
        if (!repo.transitioning) {
            // Unreleased groups need members view
            if (repo.profileType === 'group' && repo.type !== 'released') {
                views.push(self.membersView);
            }

            // Unreleased repositories need content view
            if (repo.type !== 'released' && !repo.transitioning) {
                views.push(self.contentView);
            }
        }

        iconCls = Nexus.staging.Icons.forRepository(repo).cls;
        self.setTitle(repo.repositoryId, iconCls);

        // Update enabled tabs with the currently selected data
        Ext.each(views, function (view) {
            view.updateRecord(repo);
        });

        // Change the view set
        self.enableViews(views);

        self.logGroupEnd();
    }
});